﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.ComponentModel;
using System.Windows.Input;
using Infragistics.Windows;
using Infragistics.Windows.Helpers;
using Infragistics.Windows.Editors;
using System.Windows.Controls;

namespace Infragistics.Windows.Samples
{
    /// <summary>
    /// Custom <see cref="ValueEditor"/> used to host arbitrary elements.
    /// </summary>
    [TemplatePart(Name = "PART_FocusBefore", Type = typeof(FrameworkElement))]
    [TemplatePart(Name = "PART_FocusAfter", Type = typeof(FrameworkElement))]
    public class ControlHostEditor : ValueEditor
    {
        #region Member Variables

        private FrameworkElement _focusBefore;
        private FrameworkElement _focusAfter;
        private IInputElement _focusSite;

        #endregion //Member Variables

        #region Constructor
        static ControlHostEditor()
        {
            FrameworkElement.DefaultStyleKeyProperty.OverrideMetadata(typeof(ControlHostEditor), new FrameworkPropertyMetadata(typeof(ControlHostEditor)));
            Control.HorizontalContentAlignmentProperty.OverrideMetadata(typeof(ControlHostEditor), new FrameworkPropertyMetadata(KnownBoxes.HorizontalAlignmentStretchBox));
            Control.VerticalContentAlignmentProperty.OverrideMetadata(typeof(ControlHostEditor), new FrameworkPropertyMetadata(KnownBoxes.VerticalAlignmentStretchBox));
            ValueEditor.ValueTypeProperty.OverrideMetadata(typeof(ControlHostEditor), new FrameworkPropertyMetadata(typeof(object)));
        }

        /// <summary>
        /// Initializes a new <see cref="ControlHostEditor"/>
        /// </summary>
        public ControlHostEditor()
        {
			this.SetValue(EditorPropertyKey, this);
        }
        #endregion //Constructor

        #region Base class overrides

        #region Can(Render|Edit)Type
        /// <summary>
        /// Indicates if the editor can handle the specified type
        /// </summary>
        /// <param name="type">The type to evaluate</param>
        /// <returns>Always returns true since it is up to the host template to actually edit the value.</returns>
        public override bool CanEditType(Type type)
        {
            return true;
        }

        /// <summary>
        /// Indicates if the editor can handle the specified type
        /// </summary>
        /// <param name="type">The type to evaluate</param>
        /// <returns>Always returns true since it is up to the host template to actually render the value.</returns>
        public override bool CanRenderType(Type type)
        {
            return true;
        }

        #endregion //Can(Render|Edit)Type

        #region OnApplyTemplate
        /// <summary>
        /// Invoked when the template has been applied to the element.
        /// </summary>
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            _focusBefore = this.GetTemplateChild("PART_FocusBefore") as FrameworkElement;
            _focusAfter = this.GetTemplateChild("PART_FocusAfter") as FrameworkElement;

            if (this.IsEmbedded == false)
            {
                if (null != _focusBefore)
                    _focusBefore.Visibility = Visibility.Collapsed;

                if (null != _focusAfter)
                    _focusAfter.Visibility = Visibility.Collapsed;
            }
        }

        #endregion //OnApplyTemplate

        #region OnFocusSiteChanged
        /// <summary>
        /// Invoked when the FocusSite has changed for the editor.
        /// </summary>
        protected override void OnFocusSiteChanged()
        {
            base.OnFocusSiteChanged();

            if (null != _focusSite)
            {
                _focusSite.GotKeyboardFocus -= new KeyboardFocusChangedEventHandler(OnFocusSiteGotKeyboardFocus);
            }

            _focusSite = this.FocusSite as IInputElement;

            if (null != _focusSite)
            {
                _focusSite.GotKeyboardFocus += new KeyboardFocusChangedEventHandler(OnFocusSiteGotKeyboardFocus);
            }
        }
        #endregion //OnFocusSiteChanged

        #region OnKeyDown
        /// <summary>
        /// Invoked when a key is pressed within the editor.
        /// </summary>
        /// <param name="e">Provides information about the event</param>
        protected override void OnKeyDown(System.Windows.Input.KeyEventArgs e)
        {
            if (e.Key == Key.Tab && this.IsEmbedded)
            {
                bool next;

                if (Keyboard.Modifiers == ModifierKeys.None)
                    next = true;
                else if (Keyboard.Modifiers == ModifierKeys.Shift)
                    next = false;
                else
                    return;

                // this editor wants to handle the tab key to shift focus
                // to the next/previous element. if focus is shifted to
                // an element within the edit template then we want to mark 
                // the key as processed. if focus is shift to our element 
                // before/after the edit template elements then we want to 
                // allow the tab key to bubble up so the datagrid can process
                // it
                IInputElement originalFocus = Keyboard.FocusedElement;

                if (ControlHostEditor.MoveFocus(originalFocus, next))
                {
                    if (originalFocus != Keyboard.FocusedElement &&
                        Keyboard.FocusedElement != _focusBefore &&
                        Keyboard.FocusedElement != _focusAfter)
                    {
                        e.Handled = true;
                    }
                }
            }

            base.OnKeyDown(e);
        }

        #endregion //OnKeyDown

        #endregion //Base class overrides

        #region Properties

        #region HasRenderDataTemplate

        private static readonly DependencyPropertyKey HasRenderDataTemplatePropertyKey =
            DependencyProperty.RegisterReadOnly("HasRenderDataTemplate",
            typeof(bool), typeof(ControlHostEditor), new FrameworkPropertyMetadata(KnownBoxes.FalseBox));

        /// <summary>
        /// Identifies the <see cref="HasRenderDataTemplate"/> dependency property
        /// </summary>
        public static readonly DependencyProperty HasRenderDataTemplateProperty =
            HasRenderDataTemplatePropertyKey.DependencyProperty;

        /// <summary>
        /// Returns a boolean indicating if the RenderDataTemplate was set.
        /// </summary>
        /// <seealso cref="HasRenderDataTemplateProperty"/>
        [Description("Returns a boolean indicating if the RenderDataTemplate was set.")]
        [Category("Behavior")]
        [Bindable(true)]
        [ReadOnly(true)]
        public bool HasRenderDataTemplate
        {
            get
            {
                return (bool)this.GetValue(ControlHostEditor.HasRenderDataTemplateProperty);
            }
        }

        #endregion //HasRenderDataTemplate

        #region RenderDataTemplate

        /// <summary>
        /// Identifies the <see cref="RenderDataTemplate"/> dependency property
        /// </summary>
        public static readonly DependencyProperty RenderDataTemplateProperty = DependencyProperty.Register("RenderDataTemplate",
            typeof(DataTemplate), typeof(ControlHostEditor), new FrameworkPropertyMetadata(null, new PropertyChangedCallback(OnRenderDataTemplateChanged)));

        private static void OnRenderDataTemplateChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            d.SetValue(HasRenderDataTemplatePropertyKey, e.NewValue != null ? KnownBoxes.TrueBox : DependencyProperty.UnsetValue);
        }

        /// <summary>
        /// Returns or sets the template used to provide the template when in edit mode
        /// </summary>
        /// <seealso cref="RenderDataTemplateProperty"/>
        [Description("Returns or sets the template used to provide the template when in edit mode")]
        [Category("Behavior")]
        [Bindable(true)]
        public DataTemplate RenderDataTemplate
        {
            get
            {
                return (DataTemplate)this.GetValue(ControlHostEditor.RenderDataTemplateProperty);
            }
            set
            {
                this.SetValue(ControlHostEditor.RenderDataTemplateProperty, value);
            }
        }

        #endregion //RenderDataTemplate

        #region HasEditDataTemplate

        private static readonly DependencyPropertyKey HasEditDataTemplatePropertyKey =
            DependencyProperty.RegisterReadOnly("HasEditDataTemplate",
            typeof(bool), typeof(ControlHostEditor), new FrameworkPropertyMetadata(KnownBoxes.FalseBox));

        /// <summary>
        /// Identifies the <see cref="HasEditDataTemplate"/> dependency property
        /// </summary>
        public static readonly DependencyProperty HasEditDataTemplateProperty =
            HasEditDataTemplatePropertyKey.DependencyProperty;

        /// <summary>
        /// Returns a boolean indicating if the EditDataTemplate was set.
        /// </summary>
        /// <seealso cref="HasEditDataTemplateProperty"/>
        [Description("Returns a boolean indicating if the EditDataTemplate was set.")]
        [Category("Behavior")]
        [Bindable(true)]
        [ReadOnly(true)]
        public bool HasEditDataTemplate
        {
            get
            {
                return (bool)this.GetValue(ControlHostEditor.HasEditDataTemplateProperty);
            }
        }

        #endregion //HasEditDataTemplate

        #region EditDataTemplate

        /// <summary>
        /// Identifies the <see cref="EditDataTemplate"/> dependency property
        /// </summary>
        public static readonly DependencyProperty EditDataTemplateProperty = DependencyProperty.Register("EditDataTemplate",
            typeof(DataTemplate), typeof(ControlHostEditor), new FrameworkPropertyMetadata(null, new PropertyChangedCallback(OnEditDataTemplateChanged)));

        private static void OnEditDataTemplateChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            d.SetValue(HasEditDataTemplatePropertyKey, e.NewValue != null ? KnownBoxes.TrueBox : DependencyProperty.UnsetValue);
        }

        /// <summary>
        /// Returns or sets the template used to provide the template when in edit mode
        /// </summary>
        /// <seealso cref="EditDataTemplateProperty"/>
        [Description("Returns or sets the template used to provide the template when in edit mode")]
        [Category("Behavior")]
        [Bindable(true)]
        public DataTemplate EditDataTemplate
        {
            get
            {
                return (DataTemplate)this.GetValue(ControlHostEditor.EditDataTemplateProperty);
            }
            set
            {
                this.SetValue(ControlHostEditor.EditDataTemplateProperty, value);
            }
        }

        #endregion //EditDataTemplate

		#region Editor

		/// <summary>
		/// Editor Read-Only Dependency Property
		/// </summary>
		private static readonly DependencyPropertyKey EditorPropertyKey
			= DependencyProperty.RegisterAttachedReadOnly("Editor", typeof(ControlHostEditor), typeof(ControlHostEditor),
				new FrameworkPropertyMetadata((ControlHostEditor)null,
					FrameworkPropertyMetadataOptions.Inherits | FrameworkPropertyMetadataOptions.OverridesInheritanceBehavior));

		public static readonly DependencyProperty EditorProperty
			= EditorPropertyKey.DependencyProperty;

		/// <summary>
		/// Returns the containing <see cref="ControlHostEditor"/>
		/// </summary>
		/// <param name="d">The object whose containing editor is to be returned.</param>
		/// <returns>The containing editor or null</returns>
		public static ControlHostEditor GetEditor(DependencyObject d)
		{
			return (ControlHostEditor)d.GetValue(EditorProperty);
		}

		#endregion  //Editor

        #endregion //Properties

        #region Methods

        #region MoveFocus
        private static bool MoveFocus(IInputElement src, bool next)
        {
            return MoveFocus(src, next ? FocusNavigationDirection.Next : FocusNavigationDirection.Previous);
        }

        private static bool MoveFocus(IInputElement src, FocusNavigationDirection direction)
        {
            FrameworkElement fe = src as FrameworkElement;
            TraversalRequest req = new TraversalRequest(direction);

            if (null != fe)
                return fe.MoveFocus(req);

            FrameworkContentElement fce = src as FrameworkContentElement;

            if (null != fce)
                return fce.MoveFocus(req);

            return false;
        }
        #endregion //MoveFocus

        #region OnFocusSiteGotKeyboardFocus
        private void OnFocusSiteGotKeyboardFocus(object sender, KeyboardFocusChangedEventArgs e)
        {
            if (e.NewFocus == _focusSite)
            {
                // this isn't ideal but if someone shift tab's into the edit then focus
                // the last child element
                FocusNavigationDirection dir = Keyboard.IsKeyDown(Key.Tab) && Keyboard.Modifiers == ModifierKeys.Shift
                    ? FocusNavigationDirection.Last
                    : FocusNavigationDirection.First;

                MoveFocus(_focusSite, dir);
            }
        }
        #endregion //OnFocusSiteGotKeyboardFocus

        #endregion //Methods
    }
}
